#############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""tests

$Id: test_directoryresource.py 95447 2009-01-29 16:28:18Z wosc $
"""
from z3c.hashedresource import interfaces
import os
import re
import shutil
import tempfile
import z3c.hashedresource.tests
import zope.app.testing.functional
import zope.publisher.browser
import zope.security.checker
import zope.site


fixture = os.path.join(
    os.path.dirname(z3c.hashedresource.tests.__file__), 'fixture')

checker = zope.security.checker.NamesChecker(
    ('get', '__getitem__', 'request', 'publishTraverse')
    )

HashedResourcesLayer = zope.app.testing.functional.ZCMLLayer(
    os.path.join(os.path.dirname(__file__), 'ftesting-devmode.zcml'),
    __name__, 'HashedResourcesLayer', allow_teardown=True)


class TestRequest(zope.publisher.browser.TestRequest):

    zope.interface.implements(interfaces.IHashedResourceSkin)


class FunctionalTestCase(zope.app.testing.functional.FunctionalTestCase):

    layer = HashedResourcesLayer

    def assertMatches(self, regex, text):
        self.assert_(re.match(regex, text), "/%s/ did not match '%s'" % (
            regex, text))

    def setUp(self):
        super(FunctionalTestCase, self).setUp()
        self.site = zope.site.hooks.getSite()

        self.tmpdir = tempfile.mkdtemp()
        open(os.path.join(self.tmpdir, 'example.txt'), 'w').write('')
        self.dirname = os.path.basename(self.tmpdir)

        self.request = TestRequest()
        self.request._vh_root = self.site
        self.directory = zope.app.publisher.browser.directoryresource.DirectoryResourceFactory(
            self.tmpdir, checker, self.dirname)(self.request)
        self.directory.__parent__ = self.site

    def tearDown(self):
        super(FunctionalTestCase, self).tearDown()
        shutil.rmtree(self.tmpdir)

    def _hash(self, text):
        return re.match('http://127.0.0.1/\+\+noop\+\+([^/]*)/.*', text).group(1)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""tests

$Id: test_directoryresource.py 95447 2009-01-29 16:28:18Z wosc $
"""
from z3c.hashedresource import hash, interfaces, testing
import os
import unittest
import zope.app.publisher.browser.directoryresource
import zope.app.publisher.browser.tests
import zope.app.publisher.testing
import zope.app.testing.functional


class HashingURLTest(testing.FunctionalTestCase):

    def test_directory_url_should_contain_hash(self):
        self.assertMatches(
            'http://127.0.0.1/\+\+noop\+\+[^/]*/@@/%s' % self.dirname, self.directory())

    def test_file_url_should_contain_hash(self):
        file = zope.app.publisher.browser.fileresource.FileResourceFactory(
            os.path.join(testing.fixture, 'test.txt'), testing.checker, 'test.txt')(self.request)
        self.assertMatches(
            'http://127.0.0.1/\+\+noop\+\+[^/]*/@@/test.txt', file())

    def test_different_files_hashes_should_differ(self):
        file1 = zope.app.publisher.browser.fileresource.FileResourceFactory(
            os.path.join(testing.fixture, 'test.txt'), testing.checker, 'test.txt')(self.request)
        file2 = zope.app.publisher.browser.fileresource.FileResourceFactory(
            os.path.join(testing.fixture, 'test.pt'), testing.checker, 'test.txt')(self.request)
        self.assertNotEqual(self._hash(file1()), self._hash(file2()))

    def test_directory_contents_changed_hash_should_change(self):
        before = self._hash(self.directory())
        open(os.path.join(self.tmpdir, 'example.txt'), 'w').write('foo')
        after = self._hash(self.directory())
        self.assertNotEqual(before, after)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(HashingURLTest))
    return suite



#############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""tests

$Id: test_directoryresource.py 95447 2009-01-29 16:28:18Z wosc $
"""
from z3c.hashedresource import interfaces, testing
import unittest
import zope.app.testing.functional
import zope.testbrowser.testing


class BrowserTest(zope.app.testing.functional.FunctionalTestCase):

    layer = testing.HashedResourcesLayer

    def setUp(self):
        super(BrowserTest, self).setUp()
        self.browser = zope.testbrowser.testing.Browser()
        self.directory = zope.component.getAdapter(
            zope.publisher.browser.TestRequest(), name='myresource')

    def test_traverse_atat_by_name(self):
        self.browser.open('http://localhost/@@/myresource/test.txt')
        self.assertEqual('test\ndata\n', self.browser.contents)

    def test_traverse_atat_by_hash(self):
        hash = str(
            interfaces.IResourceContentsHash(self.directory))
        self.browser.open(
            'http://localhost/++noop++%s/@@/myresource/test.txt' % hash)
        self.assertEqual('test\ndata\n', self.browser.contents)

    def test_traverse_resource_by_name(self):
        self.browser.open('http://localhost/++resource++myresource/test.txt')
        self.assertEqual('test\ndata\n', self.browser.contents)

    def test_traverse_resource_by_hash(self):
        hash = str(
            interfaces.IResourceContentsHash(self.directory))
        self.browser.open(
            'http://localhost/++noop++%s/++resource++myresource/test.txt' % hash)
        self.assertEqual('test\ndata\n', self.browser.contents)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(BrowserTest))
    return suite



#############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from z3c.hashedresource import interfaces
from zope.component import adapts
from zope.interface import implements, implementsOnly, directlyProvides
from zope.traversing.browser.interfaces import IAbsoluteURL
import zope.app.publisher.browser.resource
import zope.publisher.interfaces.browser
import zope.traversing.browser.absoluteurl
try:
    from zope.app.publisher.interfaces import IResource
except ImportError:
    from zope.component.interfaces import IResource


class HashingURL(zope.traversing.browser.absoluteurl.AbsoluteURL):
    """Inserts a hash of the contents into the resource's URL,
    so the URL changes whenever the contents change, thereby forcing
    a browser to update its cache.
    """

    implementsOnly(IAbsoluteURL)
    adapts(IResource, interfaces.IHashedResourceSkin)

    def __init__(self, context, request):
        self.context = context
        self.request = request

        # XXX duplicated from zope.app.publisher.browser.resource.AbsoluteURL
        self.name = self.context.__name__
        if self.name.startswith('++resource++'):
            self.name = self.name[12:]

    def _site_url(self):
        url = zope.app.publisher.browser.resource.AbsoluteURL(
            self.context, self.request)()
        return url.split('/@@/')[0]

    def __str__(self):
        hash = str(interfaces.IResourceContentsHash(self.context))
        return "%s/++noop++%s/@@/%s" % (self._site_url(), hash, self.name)



#############################################################################
#
# Copyright (c) 2009 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""

$Id$
"""
from zope.interface import Interface, alsoProvides
import zope.publisher.interfaces.browser


class IResourceContentsHash(Interface):

    def __str__():
        """return a hash of the contents of the resource"""


class IHashedResourceSkin(
    zope.publisher.interfaces.browser.IDefaultBrowserLayer):
    """marker interface to differentiate our AbsoluteURL registration
    from the default one in zope.app.publisher."""

alsoProvides(
    IHashedResourceSkin, zope.publisher.interfaces.browser.IBrowserSkinType)



#############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from z3c.hashedresource import interfaces
from zope.interface import implements, implementsOnly
import hashlib
import os


class ContentsHash(object):

    implements(interfaces.IResourceContentsHash)

    def __init__(self, context):
        self.context = context

    def __str__(self):
        path = self.context.context.path
        if os.path.isdir(path):
            files = self._list_directory(path)
        else:
            files = [path]

        result = hashlib.new('md5')
        for file in files:
            f = open(file, 'rb')
            data = f.read()
            f.close()
            result.update(data)
        result = result.hexdigest()
        return result

    def _list_directory(self, path):
        for root, dirs, files in os.walk(path):
            for file in files:
                yield os.path.join(root, file)


_contents_hash = {}

class CachingContentsHash(ContentsHash):

    def __str__(self):
        path = self.context.context.path
        try:
            return _contents_hash[path]
        except KeyError:
            result = super(CachingContentsHash, self).__str__()
            _contents_hash[path] = result
            return result



import os
from setuptools import setup, find_packages


setup(
    name='z3c.hashedresource',
    version = '1.1.1',
    author='Wolfgang Schnerring',
    author_email='ws@gocept.com',
    description='Provides URLs for resources that change whenever their content changes.',
    url='http://pypi.python.org/pypi/z3c.hashedresource',
    long_description= (
        open(os.path.join('src', 'z3c', 'hashedresource', 'README.txt')).read()
        + '\n\n'
        + open('CHANGES.txt').read()),
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Framework :: Zope3'],
    license='ZPL 2.1',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    namespace_packages=['z3c'],
    install_requires=[
        'setuptools',
        'z3c.noop',
        # When using Zope3, we need zope.app.publisher>=3.8.2.
        'zope.app.publisher',
        'zope.component',
        'zope.interface',
        'zope.publisher',
        ],
    extras_require=dict(
        test=[
            'zope.app.testing',
            'zope.app.zcmlfiles',
            'zope.security',
            'zope.site',
            'zope.testbrowser',
        ]),
    include_package_data = True,
    zip_safe = False,
)



#############################################################################
#
# Copyright (c) 2006-2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""tests

$Id: test_directoryresource.py 95447 2009-01-29 16:28:18Z wosc $
"""
from z3c.hashedresource import hash, testing
import os
import unittest
import zope.component


ProductionModeLayer = zope.app.testing.functional.ZCMLLayer(
    os.path.join(os.path.dirname(testing.__file__), 'ftesting.zcml'),
    __name__, 'ProductionModeLayer', allow_teardown=True)


class CachingContentsHashTest(testing.FunctionalTestCase):

    layer = ProductionModeLayer

    def test_production_mode_hash_should_not_change(self):
        zope.component.provideAdapter(
            hash.CachingContentsHash,
            (zope.app.publisher.browser.directoryresource.DirectoryResource,))

        before = self._hash(self.directory())
        open(os.path.join(self.tmpdir, 'example.txt'), 'w').write('foo')
        after = self._hash(self.directory())
        self.assertEqual(before, after)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CachingContentsHashTest))
    return suite
